package net.sf.sql2java.common.store;

import java.util.List;

import net.sf.sql2java.common.beans.Processable;

public interface ProcessorStore {

// -- Processables by Scope ---------------------------------------------------
	
	/**
	 * Get the processables for the given scope.
	 * 
	 * @param scope		The scope the processable's must be a part of
	 * @return 	a list of processable's which will be used in the given scope.
	 * 			if no processable's can be found, an empty list will be 
	 * 			returned.
	 */
	public List<Processable> getProcessables(String scope);

	public List<Processable> getProcessables(String scope, String parentKey);
	
// -- Processable Manipulation ------------------------------------------------
	
	/**
	 * Add a processable to the internal structure.
	 * 
	 * @param processable	The processable to add
	 */
	public void addProcessable(Processable processable) throws StoreException;
		
	/**
	 * Remove a processable from the internal structure.
	 * 
	 * @param processable	The processable to add
	 */
	public void removeProcessable(Processable processable);
	
	/**
	 * Create a duplicate of the given processable.
	 * 
	 * @param processable	The processable to duplicate
	 * @return	A duplicate of the given processable. Changing attributes on
	 * 			this new processable won't affect the old one
	 */
	public Processable duplicateProcessable(Processable processable);
	
	/**
	 * Save changes made to the given processable
	 * 
	 * @param processable	The processable to persist
	 */
	public void save(Processable processable) throws StoreException;
	
	/**
	 * Get the processable with the given key
	 * 
	 * @param key 	The processable's key
	 * @return 	the processable with the given key, or 'null' if no 
	 * 			processable with the given key could be found
	 */
	public Processable getProcessable(String key);
	
	/**
	 * Check whether a processable with the given key exists.
	 * 
	 * @param key	The processable's key
	 * @return	true if there is a processable with the given key available in
	 * 			this store, false if there is none
	 */
	public boolean hasProcessable(String key);
}
