/*
 * PackageReader.java
 * 
 * Created on Jul 11, 2007, 12:51:08 PM
 * 
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */
package net.sf.sql2java.generator.template.pack;

import java.io.File;
import java.io.IOException;

import net.sf.sql2java.common.parameters.StringParser;
import net.sf.sql2java.common.parameters.handlers.ContextParameterHandler;
import net.sf.sql2java.generator.scripting.Script;
import net.sf.sql2java.generator.template.Template;

import org.apache.commons.configuration.Configuration;
import org.apache.commons.configuration.ConfigurationException;
import org.apache.commons.configuration.XMLConfiguration;

/**
 *
 * @author dge2
 */
public abstract class PackageReader {
    
    public abstract void readPack(TemplatePack result, String packPath) throws IOException, ConfigurationException;
    
    protected void readDescriptor(TemplatePack result, File file) throws ConfigurationException {
    	// -- create the descriptor configuration
    	XMLConfiguration descriptor = new XMLConfiguration(file);

        // -- read the descriptor and create the result object
        readTemplatePack(result, descriptor);
    }
    
    protected void readTemplatePack(TemplatePack result, Configuration config) {
    	// -- read the name, version and description
    	result.setDescription(config.getString("description"));
    	result.setVersion(config.getString("version"));
    	result.setName(config.getString("name"));
    	
    	// -- read the properties
		readProperties(result, config.subset("properties"));
    	
    	// -- read the templates
		readTemplates(result, config.subset("templates"));
    	
    	// -- read the scripts
		readScripts(result, config.subset("scripts"));
    }
    
    /**
     * Read all the properties from the given configuration.
     * 
     * @param pack		The template pack to which we need to add the properties
     * @param config	A Configuration object holding the properties
     */
    protected void readProperties(TemplatePack pack, Configuration config) {
    	// -- determine the amount of templates
    	int size = config.getList("property[@name]").size();
    	
    	// -- read the templates
    	for (int i = 0; i < size; i++)
    		readProperty(pack, config.subset(String.format("property(%d)", i)));
    }
    
    /**
     * Read a single property from the given configuration.
     * 
     * @param pack		The template pack to which we need to add the property
     * @param config	A Configuration object holding the property
     */
    protected void readProperty(TemplatePack pack, Configuration config) {
    	// -- get the property name and value
    	String name = config.getString("[@name]");
    	
    	// -- get the value if it's already stored inside the result
    	String value = pack.getProperties().get(name);
    	
    	// -- set the default if no value has been found
    	if (value == null)
    		value = config.getString("[@default]");
    	
    	// -- parse the property
    	StringParser parser = new StringParser(new ContextParameterHandler(pack.getProperties()));
    	
    	// -- insert the property into the template pack
    	pack.getProperties().put(name, parser.parse(value));
    }
    
    /**
     * Read all the templates from the given configuration
     * 
     * @param pack		The template pack to which we need to add the templates
     * @param config	A Configuration object holding the templates
     */
    protected void readTemplates(TemplatePack pack, Configuration config) {
    	// -- determine the amount of templates
    	int size = config.getList("template[@name]").size();
    	
    	// -- read the templates
    	for (int i = 0; i < size; i++)
    		readTemplate(pack, config.subset(String.format("template(%d)", i)));
    }
    
    /**
     * Read a single template from the configuration and put it into the template pack.
     * 
     * @param pack		The template pack to which we need to add the template
     * @param config	A Configuration object holding the template configuration
     */
    protected void readTemplate(TemplatePack pack, Configuration config) {
    	// -- create the result
    	Template result = new Template();
    	
    	// -- Create the StringParser which will be used to read this pack's 
    	// -- configuration
    	StringParser parser = new StringParser(new ContextParameterHandler(pack.getProperties()));
    	
    	// -- get the template values
    	result.setName(config.getString("[@name]"));
    	result.setResource(parser.parse(config.getString("[@resource]")));
    	result.setScope(config.getString("[@scope]").toLowerCase());
    	
    	result.setPath(parser.parse(config.getString("target[@path]")));
    	result.setEntityName(parser.parse(config.getString("target[@entityname]")));
    	result.setExtension(parser.parse(config.getString("target[@extension]")));
    	
    	// -- add the result to the templatePack
    	pack.addTemplate(result);
    }
    
    /**
     * Read all the scripts from the given configuration.
     * 
     * @param pack		The template pack to which we need to add the scripts
     * @param config	A Configuration object holding the scripts configuration
     */
    protected void readScripts(TemplatePack pack, Configuration config) {
    	// -- determine the amount of templates
    	int size = config.getList("script[@name]").size();
    	
    	// -- read the templates
    	for (int i = 0; i < size; i++)
    		readScript(pack, config.subset(String.format("script(%d)", i)));
    }
    
    /**
     * Read a single script from the given configuration.
     * 
     * @param pack		The template pack to which we need to add the script
     * @param config	A Configuration object holding the script configuration
     */
    protected void readScript(TemplatePack pack, Configuration config) {
    	// -- create the result
    	Script result = new Script();
    	
    	// -- Create the StringParser which will be used to read this pack's 
    	// -- configuration
    	StringParser parser = new StringParser(new ContextParameterHandler(pack.getProperties()));
    	
    	// -- get the script values
    	result.setName(config.getString("[@name]"));
    	result.setResource(parser.parse(config.getString("[@resource]")));
    	result.setEngineType(config.getString("[@engine]"));
    	
    	// -- add the script to the pack
    	pack.addScript(result);
    }
}
