package net.sf.sql2java.generator.writer;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.StringReader;

import net.sf.sql2java.common.parameters.StringParser;
import net.sf.sql2java.common.parameters.handlers.ModifierParameterHandler;

/**
 * Write the files with regards to usercode.
 *
 * If the file contains certain code tags, this won't be overwritten.
 *
 * @author daang
 */
public class CheckedCodeWriter extends CodeWriter {
	private StringParser parser = new StringParser(new ModifierParameterHandler());

// -- Writing -----------------------------------------------------------------
    
    public void write(String path, String content) throws Exception {
        // -- we need to add the target directory to the actual path
        //path = appendTargetDirectory(path);
        
        // -- create a file for the path
        File file = new File(path);
        
        // -- invoke the modifiers
        content = invokeModifiers(content);
        
        // -- inject usercode
        content = injectUserCode(file, content);
        
        // -- wrap the file into a writer so we can put data in it
        PrintWriter writer = new PrintWriter(new OutputStreamWriter(new FileOutputStream(file)));

        // -- write our content to the file
        writer.write(content);
        writer.flush();
        writer.close();
    }
    
    public String invokeModifiers(String content) {
    	// -- return the result
    	return parser.parse(content);
    }
    
    public String injectUserCode(File file, String content) {
        // -- create the result object
        String result = content;
        
        try {
            // -- first check the filetype
            //String type = getExtension(file.getName());
        
            // -- create the merger
            UsercodeMerger merger = new UsercodeMerger();
        
            // -- if the file does not exists, create the directories
            if (! file.getParentFile().exists()) {
                boolean succes = file.getParentFile().mkdirs();
            
                // -- check if the creation of the directories worked
                if (! succes)
                	throw new IOException("Unable to create the directory to write the file to");
            }
            
            // -- and start merging
            if (merger != null)
                result = merger.mergeUsercode(
                        new StringReader(content), 
                        (file.exists()) ? new FileReader(file) : null
                );
            
        } catch (FileNotFoundException ex) {
            ex.printStackTrace();
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        
        // -- return the result object
        return result;
    }
}
